def pack_padded_sequence(input: Tensor,
    lengths: Tensor,
    batch_first: bool=False,
    enforce_sorted: bool=True) -> __torch__.torch.nn.utils.rnn.PackedSequence:
  _0 = __torch__.torch.nn.utils.rnn._packed_sequence_init
  lengths0 = torch.as_tensor(lengths, dtype=4)
  if enforce_sorted:
    input0, lengths1, sorted_indices = input, lengths0, None
  else:
    lengths2, sorted_indices0 = torch.sort(lengths0, -1, True)
    sorted_indices1 = torch.to(sorted_indices0, ops.prim.device(input))
    if batch_first:
      batch_dim = 0
    else:
      batch_dim = 1
    input1 = torch.index_select(input, batch_dim, sorted_indices1)
    input0, lengths1, sorted_indices = input1, lengths2, sorted_indices1
  data, batch_sizes = torch._pack_padded_sequence(input0, lengths1, batch_first)
  _1 = _0(data, batch_sizes, sorted_indices, None, )
  return _1
def pad_packed_sequence(sequence: __torch__.torch.nn.utils.rnn.PackedSequence,
    batch_first: bool=False,
    padding_value: float=0.,
    total_length: Optional[int]=None) -> Tuple[Tensor, Tensor]:
  _2 = "Expected total_length to be at least the length of the longest sequence in input, but got total_length={} and max sequence length being {}"
  max_seq_length = torch.size((sequence)[1], 0)
  if torch.__isnot__(total_length, None):
    total_length0 = unchecked_cast(int, total_length)
    _3 = torch.lt(total_length0, max_seq_length)
    if _3:
      _4 = torch.format(_2, total_length0, max_seq_length)
      ops.prim.RaiseException(_4)
    else:
      pass
    max_seq_length0 = total_length0
  else:
    max_seq_length0 = max_seq_length
  padded_output, lengths = torch._pad_packed_sequence((sequence)[0], (sequence)[1], batch_first, padding_value, max_seq_length0)
  unsorted_indices = (sequence)[3]
  _5 = torch.__isnot__(unsorted_indices, None)
  if _5:
    unsorted_indices0 = unchecked_cast(Tensor, unsorted_indices)
    if batch_first:
      batch_dim = 0
    else:
      batch_dim = 1
    _7 = torch.index_select(padded_output, batch_dim, unsorted_indices0)
    _8 = annotate(List[Optional[Tensor]], [unsorted_indices0])
    _6 = (_7, torch.index(lengths, _8))
  else:
    _6 = (padded_output, lengths)
  return _6
class PackedSequence(NamedTuple):
  data : Tensor
  batch_sizes : Tensor
  sorted_indices : Optional[Tensor]
  unsorted_indices : Optional[Tensor]
def _packed_sequence_init(data: Tensor,
    batch_sizes: Optional[Tensor]=None,
    sorted_indices: Optional[Tensor]=None,
    unsorted_indices: Optional[Tensor]=None) -> __torch__.torch.nn.utils.rnn.PackedSequence:
  _9 = __torch__.torch.nn.utils.rnn._packed_sequence_init_args
  _10 = _9(data, batch_sizes, sorted_indices, unsorted_indices, )
  data0, batch_sizes0, sorted_indices2, unsorted_indices1, = _10
  _11 = __torch__.torch.nn.utils.rnn.PackedSequence(data0, batch_sizes0, sorted_indices2, unsorted_indices1)
  return _11
def _packed_sequence_init_args(data: Tensor,
    batch_sizes: Optional[Tensor]=None,
    sorted_indices: Optional[Tensor]=None,
    unsorted_indices: Optional[Tensor]=None) -> Tuple[Tensor, Tensor, Optional[Tensor], Optional[Tensor]]:
  _12 = __torch__.torch.nn.utils.rnn.invert_permutation
  _13 = "batch_sizes should always be on CPU. Instances of PackedSequence should never be created manually. They should be instantiated by functions like pack_sequence and pack_padded_sequences in nn.utils.rnn. https://pytorch.org/docs/stable/nn.html#torch.nn.utils.rnn.pack_sequence"
  _14 = uninitialized(Tuple[Tensor, Tensor, Optional[Tensor], Optional[Tensor]])
  if torch.__is__(unsorted_indices, None):
    unsorted_indices2 : Optional[Tensor] = _12(sorted_indices, )
  else:
    unsorted_indices3 = unchecked_cast(Tensor, unsorted_indices)
    unsorted_indices2 = unsorted_indices3
  if torch.__isnot__(batch_sizes, None):
    batch_sizes1 = unchecked_cast(Tensor, batch_sizes)
    _16 = ops.prim.type(ops.prim.device(batch_sizes1))
    if torch.ne(_16, "cpu"):
      ops.prim.RaiseException(_13)
    else:
      pass
    _17 = (data, batch_sizes1, sorted_indices, unsorted_indices2)
    _15 = _17
  else:
    ops.prim.RaiseException("AssertionError: ")
    _15 = _14
  return _15
def invert_permutation(permutation: Optional[Tensor]) -> Optional[Tensor]:
  if torch.__is__(permutation, None):
    _18 : Optional[Tensor] = None
  else:
    permutation0 = unchecked_cast(Tensor, permutation)
    output = torch.empty_like(permutation0, dtype=None, layout=None, device=None, pin_memory=None, memory_format=0)
    _19 = torch.arange(0, torch.numel(permutation0), dtype=None, layout=None, device=ops.prim.device(permutation0))
    _20 = torch.scatter_(output, 0, permutation0, _19)
    _18 = output
  return _18
